@@ -33,9 +33,13 @@
  */
 
 /*
- * $Id: carray.c,v 1.9 2005/06/01 12:21:57 smarinier Exp $
+ * $Id: carray.c,v 1.10 2006/05/22 13:39:40 hoa Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#	include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include "carray.h"
