@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailstream_low.c,v 1.17 2006/05/22 13:39:40 hoa Exp $
+ * $Id: mailstream_low.c,v 1.18 2006/12/22 18:45:44 hoa Exp $
  */
 
 #include "mailstream_low.h"
@@ -184,3 +184,14 @@ ssize_t mailstream_low_write(mailstream_low * s,
 
   return s->driver->mailstream_write(s, buf, count);
 }
+
+void mailstream_low_cancel(mailstream_low * s)
+{
+  if (s == NULL)
+    return;
+  
+  if (s->driver->mailstream_cancel == NULL)
+    return;
+  
+  s->driver->mailstream_cancel(s);
+}
