@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailstream_socket.c,v 1.22 2006/06/26 11:50:26 hoa Exp $
+ * $Id: mailstream_socket.c,v 1.24 2006/12/22 18:45:44 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,6 +65,13 @@
 #	endif
 #endif
 
+#include "mailstream_cancel.h"
+
+struct mailstream_socket_data {
+  int fd;
+  struct mailstream_cancel * cancel;
+};
+
 /* mailstream_low, socket */
 
 static int mailstream_low_socket_close(mailstream_low * s);
@@ -74,22 +81,15 @@ static ssize_t mailstream_low_socket_write(mailstream_low * s,
 					   const void * buf, size_t count);
 static void mailstream_low_socket_free(mailstream_low * s);
 static int mailstream_low_socket_get_fd(mailstream_low * s);
+static void mailstream_low_socket_cancel(mailstream_low * s);
 
 static mailstream_low_driver local_mailstream_socket_driver = {
-#if ! defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
-	/* SEB */
-	mailstream_low_socket_read,
-	mailstream_low_socket_write,
-	mailstream_low_socket_close,
-	mailstream_low_socket_get_fd,
-	mailstream_low_socket_free
-#else
-  mailstream_read: mailstream_low_socket_read,
-  mailstream_write: mailstream_low_socket_write,
-  mailstream_close: mailstream_low_socket_close,
-  mailstream_free: mailstream_low_socket_free,
-  mailstream_get_fd: mailstream_low_socket_get_fd,
-#endif
+  /* mailstream_read */ mailstream_low_socket_read,
+  /* mailstream_write */ mailstream_low_socket_write,
+  /* mailstream_close */ mailstream_low_socket_close,
+  /* mailstream_get_fd */ mailstream_low_socket_get_fd,
+  /* mailstream_free */ mailstream_low_socket_free,
+  /* mailstream_cancel */ mailstream_low_socket_cancel,
 };
 
 mailstream_low_driver * mailstream_socket_driver =
@@ -106,15 +106,21 @@ static struct mailstream_socket_data * socket_data_new(int fd)
     goto err;
   
   socket_data->fd = fd;
-
+  socket_data->cancel = mailstream_cancel_new();
+  if (socket_data->cancel == NULL)
+    goto free;
+  
   return socket_data;
   
+ free:
+  free(socket_data);
  err:
   return NULL;
 }
 
 static void socket_data_free(struct mailstream_socket_data * socket_data)
 {
+  mailstream_cancel_free(socket_data->cancel);
   free(socket_data);
 }
 
@@ -186,35 +192,53 @@ static ssize_t mailstream_low_socket_read(mailstream_low * s,
   struct mailstream_socket_data * socket_data;
   
   socket_data = (struct mailstream_socket_data *) s->data;
+
+  if (mailstream_cancel_cancelled(socket_data->cancel))
+    return -1;
   
   /* timeout */
   {
     fd_set fds_read;
+#if 0
     fd_set fds_excp;
+#endif
     struct timeval timeout;
     int r;
+    int fd;
+    int max_fd;
     
     timeout = mailstream_network_delay;
     
     FD_ZERO(&fds_read);
     FD_SET(socket_data->fd, &fds_read);
+    fd = mailstream_cancel_get_fd(socket_data->cancel);
+    FD_SET(fd, &fds_read);
+#if 0
     FD_ZERO(&fds_excp);
     FD_SET(socket_data->fd, &fds_excp);
-    r = select(socket_data->fd + 1, &fds_read, NULL, &fds_excp, &timeout);
+#endif
+    max_fd = socket_data->fd;
+    if (fd > max_fd)
+      max_fd = fd;
+    r = select(max_fd + 1, &fds_read, NULL,/* &fds_excp*/ NULL, &timeout);
     if (r == 0)
       return -1;
+    
+    if (FD_ISSET(fd, &fds_read)) {
+      /* cancelled */
+      mailstream_cancel_ack(socket_data->cancel);
+      return -1;
+    }
+    
+#if 0
     if (FD_ISSET(socket_data->fd, &fds_excp))
       return -1;
+#endif
     if (!FD_ISSET(socket_data->fd, &fds_read))
       return 0;
   }
   
-/* SEB why not for all os ? */
-#ifdef WIN32
   return recv(socket_data->fd, buf, count, 0);
-#else
-  return read(socket_data->fd, buf, count);
-#endif
 }
 
 static ssize_t mailstream_low_socket_write(mailstream_low * s,
@@ -223,34 +247,55 @@ static ssize_t mailstream_low_socket_write(mailstream_low * s,
   struct mailstream_socket_data * socket_data;
 
   socket_data = (struct mailstream_socket_data *) s->data;
+  
+  if (mailstream_cancel_cancelled(socket_data->cancel))
+    return -1;
+  
   /* timeout */
   {
+    fd_set fds_read;
     fd_set fds_write;
+#if 0
     fd_set fds_excp;
+#endif
     struct timeval timeout;
     int r;
+    int fd;
+    int max_fd;
     
     timeout = mailstream_network_delay;
     
+    FD_ZERO(&fds_read);
+    fd = mailstream_cancel_get_fd(socket_data->cancel);
+    FD_SET(fd, &fds_read);
     FD_ZERO(&fds_write);
     FD_SET(socket_data->fd, &fds_write);
+#if 0
     FD_ZERO(&fds_excp);
     FD_SET(socket_data->fd, &fds_excp);
-    r = select(socket_data->fd + 1, NULL, &fds_write, &fds_excp, &timeout);
+#endif
+    max_fd = socket_data->fd;
+    if (fd > max_fd)
+      max_fd = fd;
+    r = select(max_fd + 1, &fds_read, &fds_write, /*&fds_excp */ NULL, &timeout);
     if (r == 0)
       return -1;
+    
+    if (FD_ISSET(fd, &fds_read)) {
+      /* cancelled */
+      mailstream_cancel_ack(socket_data->cancel);
+      return -1;
+    }
+    
+#if 0
     if (FD_ISSET(socket_data->fd, &fds_excp))
       return -1;
+#endif
     if (!FD_ISSET(socket_data->fd, &fds_write))
       return 0;
   }
   
-/* SEB (bis) why not for all os? */
-#ifdef WIN32
   return send(socket_data->fd, buf, count, 0);
-#else
-  return write(socket_data->fd, buf, count);
-#endif
 }
 
 
@@ -277,3 +322,10 @@ mailstream * mailstream_socket_open(int fd)
   return NULL;
 }
 
+static void mailstream_low_socket_cancel(mailstream_low * s)
+{
+  struct mailstream_socket_data * socket_data;
+
+  socket_data = (struct mailstream_socket_data *) s->data;
+  mailstream_cancel_notify(socket_data->cancel);
+}
