@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailstream_ssl.c,v 1.41 2006/09/25 17:33:03 colinleroy Exp $
+ * $Id: mailstream_ssl.c,v 1.50 2007/01/07 22:29:48 hoa Exp $
  */
 
 /*
@@ -52,6 +52,9 @@
 #ifdef HAVE_STDLIB_H
 #	include <stdlib.h>
 #endif
+#ifdef HAVE_STRING_H
+#	include <string.h>
+#endif
 #include <fcntl.h>
 
 /*
@@ -81,17 +84,21 @@
 # endif
 #endif
 
+#include "mailstream_cancel.h"
+
 #ifdef USE_SSL
 #ifndef USE_GNUTLS
 struct mailstream_ssl_data {
   int fd;
   SSL * ssl_conn;
   SSL_CTX * ssl_ctx;
+  struct mailstream_cancel * cancel;
 };
 #else
 struct mailstream_ssl_data {
   int fd;
   gnutls_session session;
+  struct mailstream_cancel * cancel;
 };
 #endif
 #endif
@@ -186,22 +193,15 @@ static ssize_t mailstream_low_ssl_write(mailstream_low * s,
 					const void * buf, size_t count);
 static void mailstream_low_ssl_free(mailstream_low * s);
 static int mailstream_low_ssl_get_fd(mailstream_low * s);
+static void mailstream_low_ssl_cancel(mailstream_low * s);
 
-#if defined (_MSC_VER) || defined (__SUNPRO_CC) || defined (__SUNPRO_C)
 static mailstream_low_driver local_mailstream_ssl_driver = {
   /* mailstream_read */ mailstream_low_ssl_read,
   /* mailstream_write */ mailstream_low_ssl_write,
   /* mailstream_close */ mailstream_low_ssl_close,
-  /* mailstream_free */ mailstream_low_ssl_free,
   /* mailstream_get_fd */ mailstream_low_ssl_get_fd,
-#else
-static mailstream_low_driver local_mailstream_ssl_driver = {
-  mailstream_read: mailstream_low_ssl_read,
-  mailstream_write: mailstream_low_ssl_write,
-  mailstream_close: mailstream_low_ssl_close,
-  mailstream_free: mailstream_low_ssl_free,
-  mailstream_get_fd: mailstream_low_ssl_get_fd,
-#endif
+  /* mailstream_free */ mailstream_low_ssl_free,
+  /* mailstream_cancel */ mailstream_low_ssl_cancel,
 };
 
 mailstream_low_driver * mailstream_ssl_driver = &local_mailstream_ssl_driver;
@@ -211,12 +211,17 @@ mailstream_low_driver * mailstream_ssl_driver = &local_mailstream_ssl_driver;
 
 #ifdef USE_SSL
 #ifndef USE_GNUTLS
-static struct mailstream_ssl_data * ssl_data_new_full(int fd, SSL_METHOD *method)
+
+static struct mailstream_ssl_context * mailstream_ssl_context_new(SSL_CTX * open_ssl_ctx, int fd);
+static void mailstream_ssl_context_free(struct mailstream_ssl_context * ssl_ctx);
+
+static struct mailstream_ssl_data * ssl_data_new_full(int fd, SSL_METHOD * method, void (* callback)(struct mailstream_ssl_context * ssl_context, void * cb_data), void * cb_data)
 {
   struct mailstream_ssl_data * ssl_data;
   SSL * ssl_conn;
   int r;
   SSL_CTX * tmp_ctx;
+  struct mailstream_cancel * cancel;
   
   mailstream_ssl_init();
   
@@ -224,6 +229,14 @@ static struct mailstream_ssl_data * ssl_data_new_full(int fd, SSL_METHOD *method
   if (tmp_ctx == NULL)
     goto err;
   
+  if (callback != NULL) {
+    struct mailstream_ssl_context * ssl_context;
+    
+    ssl_context = mailstream_ssl_context_new(tmp_ctx, fd);
+    callback(ssl_context, cb_data);
+    mailstream_ssl_context_free(ssl_context);
+  }
+  
   ssl_conn = (SSL *) SSL_new(tmp_ctx);
   if (ssl_conn == NULL)
     goto free_ctx;
@@ -235,42 +248,51 @@ static struct mailstream_ssl_data * ssl_data_new_full(int fd, SSL_METHOD *method
   if (r <= 0)
     goto free_ssl_conn;
   
+  cancel = mailstream_cancel_new();
+  if (cancel == NULL)
+    goto free_ssl_conn;
+  
   r = mailstream_prepare_fd(fd);
   if (r < 0)
-    goto free_ssl_conn;
+    goto free_cancel;
   
   ssl_data = malloc(sizeof(* ssl_data));
   if (ssl_data == NULL)
-    goto free_ctx;
+    goto free_cancel;
   
   ssl_data->fd = fd;
   ssl_data->ssl_conn = ssl_conn;
   ssl_data->ssl_ctx = tmp_ctx;
+  ssl_data->cancel = cancel;
 
   return ssl_data;
 
- free_ctx:
-  SSL_CTX_free(tmp_ctx);
+ free_cancel:
+  mailstream_cancel_free(cancel);
  free_ssl_conn:
   SSL_free(ssl_conn);
+ free_ctx:
+  SSL_CTX_free(tmp_ctx);
  err:
   return NULL;
 }
-static struct mailstream_ssl_data * ssl_data_new(int fd)
+
+static struct mailstream_ssl_data * ssl_data_new(int fd, void (* callback)(struct mailstream_ssl_context * ssl_context, void * cb_data), void * cb_data)
 {
-  return ssl_data_new_full(fd, SSLv23_client_method());
+  return ssl_data_new_full(fd, SSLv23_client_method(), callback, cb_data);
 }
 
-static struct mailstream_ssl_data * tls_data_new(int fd)
+static struct mailstream_ssl_data * tls_data_new(int fd, void (* callback)(struct mailstream_ssl_context * ssl_context, void * cb_data), void * cb_data)
 {
-  return ssl_data_new_full(fd, TLSv1_client_method());
+  return ssl_data_new_full(fd, TLSv1_client_method(), callback, cb_data);
 }
 
 #else
-static struct mailstream_ssl_data * ssl_data_new(int fd)
+static struct mailstream_ssl_data * ssl_data_new(int fd, void (* callback)(struct mailstream_ssl_context * ssl_context, void * cb_data), void * cb_data)
 {
   struct mailstream_ssl_data * ssl_data;
   gnutls_session session;
+  struct mailstream_cancel * cancel;
   
   const int cipher_prio[] = { GNUTLS_CIPHER_AES_128_CBC,
 		  		GNUTLS_CIPHER_3DES_CBC,
@@ -313,10 +335,14 @@ static struct mailstream_ssl_data * ssl_data_new(int fd)
   if (r < 0) {
     goto free_ssl_conn;
   }
-
+  
+  cancel = mailstream_cancel_new();
+  if (cancel == NULL)
+    goto free_ssl_conn;
+  
   r = mailstream_prepare_fd(fd);
   if (r < 0)
-    goto free_ssl_conn;
+    goto free_cancel;
   
   ssl_data = malloc(sizeof(* ssl_data));
   if (ssl_data == NULL)
@@ -324,22 +350,26 @@ static struct mailstream_ssl_data * ssl_data_new(int fd)
   
   ssl_data->fd = fd;
   ssl_data->session = session;
-
+  ssl_data->cancel = cancel;
+  
   return ssl_data;
-
+  
+ free_cancel:
+  mailstream_cancel_free(cancel);
  free_ssl_conn:
   gnutls_deinit(session);
  err:
   return NULL;
 }
-static struct mailstream_ssl_data * tls_data_new(int fd)
+static struct mailstream_ssl_data * tls_data_new(int fd, void (* callback)(struct mailstream_ssl_context * ssl_context, void * cb_data), void * cb_data)
 {
-  return ssl_data_new(fd);
+  return ssl_data_new(fd, callback, cb_data);
 }
 #endif
 
 static void  ssl_data_free(struct mailstream_ssl_data * ssl_data)
 {
+  mailstream_cancel_free(ssl_data->cancel);
   free(ssl_data);
 }
 
@@ -365,16 +395,16 @@ static void  ssl_data_close(struct mailstream_ssl_data * ssl_data)
 
 #endif
 
-mailstream_low * mailstream_low_ssl_open_full(int fd, int starttls)
+static mailstream_low * mailstream_low_ssl_open_full(int fd, int starttls, void (* callback)(struct mailstream_ssl_context * ssl_context, void * cb_data), void * cb_data)
 {
 #ifdef USE_SSL
   mailstream_low * s;
   struct mailstream_ssl_data * ssl_data;
 
   if (starttls)
-    ssl_data = tls_data_new(fd);
+    ssl_data = tls_data_new(fd, callback, cb_data);
   else
-    ssl_data = ssl_data_new(fd);
+    ssl_data = ssl_data_new(fd, callback, cb_data);
 
   if (ssl_data == NULL)
     goto err;
@@ -396,12 +426,12 @@ mailstream_low * mailstream_low_ssl_open_full(int fd, int starttls)
 
 mailstream_low * mailstream_low_ssl_open(int fd)
 {
-  return mailstream_low_ssl_open_full(fd, 0);
+  return mailstream_low_ssl_open_full(fd, 0, NULL, NULL);
 }
 
 mailstream_low * mailstream_low_tls_open(int fd)
 {
-  return mailstream_low_ssl_open_full(fd, 1);
+  return mailstream_low_ssl_open_full(fd, 1, NULL, NULL);
 }
 
 #ifdef USE_SSL
@@ -434,6 +464,37 @@ static int mailstream_low_ssl_get_fd(mailstream_low * s)
   return ssl_data->fd;
 }
 
+static int wait_read(mailstream_low * s)
+{
+  fd_set fds_read;
+  struct timeval timeout;
+  int fd;
+  struct mailstream_ssl_data * ssl_data;
+  int max_fd;
+  int r;
+  
+  ssl_data = (struct mailstream_ssl_data *) s->data;
+  timeout = mailstream_network_delay;
+  
+  FD_ZERO(&fds_read);
+  fd = mailstream_cancel_get_fd(ssl_data->cancel);
+  FD_SET(fd, &fds_read);
+  FD_SET(ssl_data->fd, &fds_read);
+  max_fd = ssl_data->fd;
+  if (fd > max_fd)
+    max_fd = fd;
+  r = select(max_fd + 1, &fds_read, NULL, NULL, &timeout);
+  if (r == 0)
+    return -1;
+  if (FD_ISSET(fd, &fds_read)) {
+    /* cancelled */
+    mailstream_cancel_ack(ssl_data->cancel);
+    return -1;
+  }
+  
+  return 0;
+}
+
 #ifndef USE_GNUTLS
 static ssize_t mailstream_low_ssl_read(mailstream_low * s,
 				       void * buf, size_t count)
@@ -443,10 +504,11 @@ static ssize_t mailstream_low_ssl_read(mailstream_low * s,
 
   ssl_data = (struct mailstream_ssl_data *) s->data;
   
+  if (mailstream_cancel_cancelled(ssl_data->cancel))
+    return -1;
+  
   while (1) {
     int ssl_r;
-    fd_set fds_read;
-    struct timeval timeout;
     
     r = SSL_read(ssl_data->ssl_conn, buf, count);
     if (r > 0)
@@ -461,13 +523,9 @@ static ssize_t mailstream_low_ssl_read(mailstream_low * s,
       return r;
       
     case SSL_ERROR_WANT_READ:
-      timeout = mailstream_network_delay;
-      
-      FD_ZERO(&fds_read);
-      FD_SET(ssl_data->fd, &fds_read);
-      r = select(ssl_data->fd + 1, &fds_read, NULL, NULL, &timeout);
-      if (r == 0)
-        return -1;
+      r = wait_read(s);
+      if (r < 0)
+        return r;
       break;
       
     default:
@@ -483,12 +541,10 @@ static ssize_t mailstream_low_ssl_read(mailstream_low * s,
   int r;
 
   ssl_data = (struct mailstream_ssl_data *) s->data;
+  if (mailstream_cancel_cancelled(ssl_data->cancel))
+    return -1;
   
   while (1) {
-    int ssl_r;
-    fd_set fds_read;
-    struct timeval timeout;
-    
     r = gnutls_record_recv(ssl_data->session, buf, count);
     if (r > 0)
       return r;
@@ -499,13 +555,9 @@ static ssize_t mailstream_low_ssl_read(mailstream_low * s,
       
     case GNUTLS_E_AGAIN:
     case GNUTLS_E_INTERRUPTED:
-      timeout = mailstream_network_delay;
-      
-      FD_ZERO(&fds_read);
-      FD_SET(ssl_data->fd, &fds_read);
-      r = select(ssl_data->fd + 1, &fds_read, NULL, NULL, &timeout);
-      if (r == 0)
-        return -1;
+      r = wait_read(s);
+      if (r < 0)
+        return r;
       break;
       
     default:
@@ -515,35 +567,60 @@ static ssize_t mailstream_low_ssl_read(mailstream_low * s,
 }
 #endif
 
+static int wait_write(mailstream_low * s)
+{
+  fd_set fds_read;
+  fd_set fds_write;
+  struct timeval timeout;
+  int r;
+  int fd;
+  struct mailstream_ssl_data * ssl_data;
+  int max_fd;
+  
+  ssl_data = (struct mailstream_ssl_data *) s->data;
+  if (mailstream_cancel_cancelled(ssl_data->cancel))
+    return -1;
+  
+  timeout = mailstream_network_delay;
+  
+  FD_ZERO(&fds_read);
+  fd = mailstream_cancel_get_fd(ssl_data->cancel);
+  FD_SET(fd, &fds_read);
+  FD_ZERO(&fds_write);
+  FD_SET(ssl_data->fd, &fds_write);
+  
+  max_fd = ssl_data->fd;
+  if (fd > max_fd)
+    max_fd = fd;
+  
+  r = select(ssl_data->fd + 1, &fds_read, &fds_write, NULL, &timeout);
+  if (r == 0)
+    return -1;
+  
+  if (FD_ISSET(fd, &fds_read)) {
+    /* cancelled */
+    mailstream_cancel_ack(ssl_data->cancel);
+    return -1;
+  }
+  
+  if (!FD_ISSET(ssl_data->fd, &fds_write))
+    return 0;
+  
+  return 1;
+}
+
 #ifndef USE_GNUTLS
 static ssize_t mailstream_low_ssl_write(mailstream_low * s,
 					const void * buf, size_t count)
 {
   struct mailstream_ssl_data * ssl_data;
   int ssl_r;
-  fd_set fds_write;
-  fd_set fds_excp;
-  struct timeval timeout;
   int r;
   
   ssl_data = (struct mailstream_ssl_data *) s->data;
-  
-  timeout = mailstream_network_delay;
-  
-  FD_ZERO(&fds_write);
-  FD_SET(ssl_data->fd, &fds_write);
-  FD_ZERO(&fds_excp);
-  FD_SET(ssl_data->fd, &fds_excp);
-  
-  r = select(ssl_data->fd + 1, NULL, &fds_write, &fds_excp, &timeout);
-  if (r == 0)
-    return -1;
-  
-  if (FD_ISSET(ssl_data->fd, &fds_excp))
-    return -1;
-  
-  if (!FD_ISSET(ssl_data->fd, &fds_write))
-    return 0;
+  r = wait_write(s);
+  if (r <= 0)
+    return r;
   
   r = SSL_write(ssl_data->ssl_conn, buf, count);
   if (r > 0)
@@ -569,30 +646,12 @@ static ssize_t mailstream_low_ssl_write(mailstream_low * s,
 					const void * buf, size_t count)
 {
   struct mailstream_ssl_data * ssl_data;
-  int ssl_r;
-  fd_set fds_write;
-  fd_set fds_excp;
-  struct timeval timeout;
   int r;
   
   ssl_data = (struct mailstream_ssl_data *) s->data;
-  
-  timeout = mailstream_network_delay;
-  
-  FD_ZERO(&fds_write);
-  FD_SET(ssl_data->fd, &fds_write);
-  FD_ZERO(&fds_excp);
-  FD_SET(ssl_data->fd, &fds_excp);
-  
-  r = select(ssl_data->fd + 1, NULL, &fds_write, &fds_excp, &timeout);
-  if (r == 0)
-    return -1;
-  
-  if (FD_ISSET(ssl_data->fd, &fds_excp))
-    return -1;
-  
-  if (!FD_ISSET(ssl_data->fd, &fds_write))
-    return 0;
+  r = wait_write(s);
+  if (r <= 0)
+    return r;
   
   r = gnutls_record_send(ssl_data->session, buf, count);
   if (r > 0)
@@ -616,12 +675,18 @@ static ssize_t mailstream_low_ssl_write(mailstream_low * s,
 /* mailstream */
 
 mailstream * mailstream_ssl_open(int fd)
+{
+  return mailstream_ssl_open_with_callback(fd, NULL, NULL);
+}
+
+mailstream * mailstream_ssl_open_with_callback(int fd,
+    void (* callback)(struct mailstream_ssl_context * ssl_context, void * data), void * data)
 {
 #ifdef USE_SSL
   mailstream_low * low;
   mailstream * s;
 
-  low = mailstream_low_ssl_open(fd);
+  low = mailstream_low_ssl_open_with_callback(fd, callback, data);
   if (low == NULL)
     goto err;
 
@@ -640,11 +705,11 @@ mailstream * mailstream_ssl_open(int fd)
 #endif
 }
 
-int mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER)
+ssize_t mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER)
 {
 #ifdef USE_SSL
   struct mailstream_ssl_data *data = NULL;
-  int len = 0;
+  ssize_t len = 0;
 #ifndef USE_GNUTLS
   SSL *ssl_conn = NULL;
   X509 *cert = NULL;
@@ -652,16 +717,15 @@ int mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER)
   gnutls_session session = NULL;
   const gnutls_datum *raw_cert_list;
   unsigned int raw_cert_list_length;
-  char *exported_cert = NULL;
   gnutls_x509_crt cert = NULL;
   char output[10*1024];
+  size_t cert_size;
 #endif
 
   if (cert_DER == NULL || stream == NULL || stream->low == NULL)
     return -1;
 
   data = stream->low->data;
-  
   if (data == NULL)
     return -1;
 
@@ -669,11 +733,14 @@ int mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER)
   ssl_conn = data->ssl_conn;
   if (ssl_conn == NULL)
     return -1;
+  
   cert = SSL_get_peer_certificate(ssl_conn);
   if (cert == NULL)
     return -1;
+  
   *cert_DER = NULL;
-  len = i2d_X509(cert, cert_DER);
+  len = (ssize_t) i2d_X509(cert, cert_DER);
+  
   return len;
 #else
   session = data->session;
@@ -683,19 +750,120 @@ int mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER)
   && gnutls_certificate_type_get(session) == GNUTLS_CRT_X509
   &&  gnutls_x509_crt_init(&cert) >= 0
   &&  gnutls_x509_crt_import(cert, &raw_cert_list[0], GNUTLS_X509_FMT_DER) >= 0) {
-    len = 10*1024;
-    if (gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, output, &len) < 0)
+    cert_size = sizeof(output);
+    if (gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, output, &cert_size) < 0)
       return -1;
-    if (len >= 0) {
-      *cert_DER = malloc (len + 1);
-      if (*cert_DER == NULL)
-	return -1;
-      memcpy (*cert_DER, output, len);
-    }
+    
+    *cert_DER = malloc (cert_size + 1);
+    if (*cert_DER == NULL)
+      return -1;
+    
+    memcpy (*cert_DER, output, cert_size);
+    len = (ssize_t)cert_size;
     gnutls_x509_crt_deinit(cert);
+    
     return len;
   }
 #endif
 #endif
   return -1;
 }
+
+static void mailstream_low_ssl_cancel(mailstream_low * s)
+{
+  struct mailstream_ssl_data * data;
+  
+  data = s->data;
+  mailstream_cancel_notify(data->cancel);
+}
+
+struct mailstream_ssl_context
+{
+  int fd;
+#ifndef USE_GNUTLS
+  SSL_CTX * openssl_ssl_ctx;
+#else
+  gnutls_session gnutls_session;
+  gnutls_certificate_credentials_t gnutls_credentials;
+#endif
+};
+
+mailstream_low * mailstream_low_ssl_open_with_callback(int fd,
+    void (* callback)(struct mailstream_ssl_context * ssl_context, void * data), void * data)
+{
+  return mailstream_low_ssl_open_full(fd, 0, callback, data);
+}
+
+mailstream_low * mailstream_low_tls_open_with_callback(int fd,
+    void (* callback)(struct mailstream_ssl_context * ssl_context, void * data), void * data)
+{
+  return mailstream_low_ssl_open_full(fd, 1, callback, data);
+}
+
+int mailstream_ssl_set_client_certicate(struct mailstream_ssl_context * ssl_context,
+    char * filename)
+{
+#ifdef USE_SSL
+#ifdef USE_GNUTLS
+  /* not implemented */
+  return -1;
+#else
+  SSL_CTX * ctx = (SSL_CTX *)ssl_context->openssl_ssl_ctx;
+  STACK_OF(X509_NAME) *cert_names;
+  
+  cert_names = SSL_load_client_CA_file(filename);
+  if (cert_names != NULL) {
+    SSL_CTX_set_client_CA_list(ctx, cert_names);
+    return 0;
+  }
+  else {
+    return -1;
+  }
+#endif /* USE_GNUTLS */
+#else
+  return -1;
+#endif /* USE_SSL */
+}
+
+int mailstream_ssl_set_server_certicate(struct mailstream_ssl_context * ssl_context, 
+    char * CAfile, char * CApath)
+{
+#ifdef USE_SSL
+#ifdef USE_GNUTLS
+  /* not implemented */
+  return -1;
+#else
+  SSL_CTX * ctx = (SSL_CTX *)ssl_context->openssl_ssl_ctx;
+  SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, 0);
+  if (!SSL_CTX_load_verify_locations(ctx, CAfile, CApath))
+    return -1;
+  else
+    return 0;
+#endif /* USE_GNUTLS */
+#else
+  return -1;
+#endif /* USE_SSL */
+}
+
+#ifdef USE_SSL
+#ifndef USE_GNUTLS
+static struct mailstream_ssl_context * mailstream_ssl_context_new(SSL_CTX * open_ssl_ctx, int fd)
+{
+  struct mailstream_ssl_context * ssl_ctx;
+  
+  ssl_ctx = malloc(sizeof(* ssl_ctx));
+  if (ssl_ctx == NULL)
+    return NULL;
+  
+  ssl_ctx->openssl_ssl_ctx = open_ssl_ctx;
+  ssl_ctx->fd = fd;
+  
+  return ssl_ctx;
+}
+
+static void mailstream_ssl_context_free(struct mailstream_ssl_context * ssl_ctx)
+{
+  free(ssl_ctx);
+}
+#endif
+#endif
