@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailstream_ssl.h,v 1.12 2006/02/17 20:45:14 colinleroy Exp $
+ * $Id: mailstream_ssl.h,v 1.16 2007/01/07 22:29:48 hoa Exp $
  */
 
 #ifndef MAILSTREAM_SSL_H
@@ -49,14 +49,48 @@ extern "C" {
 extern mailstream_low_driver * mailstream_ssl_driver;
 #endif
 
+struct mailstream_ssl_context;
+
+LIBETPAN_EXPORT
 mailstream_low * mailstream_low_ssl_open(int fd);
+
+LIBETPAN_EXPORT
 mailstream_low * mailstream_low_tls_open(int fd);
+
+LIBETPAN_EXPORT
 mailstream * mailstream_ssl_open(int fd);
 
+LIBETPAN_EXPORT
+mailstream * mailstream_ssl_open_with_callback(int fd,
+    void (* callback)(struct mailstream_ssl_context * ssl_context, void * data), void * data);
+
+LIBETPAN_EXPORT
 void mailstream_gnutls_init_not_required(void);
+
+LIBETPAN_EXPORT
 void mailstream_openssl_init_not_required(void);
+
+LIBETPAN_EXPORT
 void mailstream_ssl_init_not_required(void);
-int mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER);
+
+LIBETPAN_EXPORT
+ssize_t mailstream_ssl_get_certificate(mailstream *stream, unsigned char **cert_DER);
+
+LIBETPAN_EXPORT
+mailstream_low * mailstream_low_ssl_open_with_callback(int fd,
+    void (* callback)(struct mailstream_ssl_context * ssl_context, void * data), void * data);
+
+LIBETPAN_EXPORT
+mailstream_low * mailstream_low_tls_open_with_callback(int fd,
+    void (* callback)(struct mailstream_ssl_context * ssl_context, void * data), void * data);
+
+LIBETPAN_EXPORT
+int mailstream_ssl_set_client_certicate(struct mailstream_ssl_context * ssl_context,
+    char * file_name);
+
+LIBETPAN_EXPORT
+int mailstream_ssl_set_server_certicate(struct mailstream_ssl_context * ssl_context, 
+    char * CAfile, char * CApath);
 
 #ifdef __cplusplus
 }
