@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: imapdriver.c,v 1.55 2006/06/26 11:50:26 hoa Exp $
+ * $Id: imapdriver.c,v 1.56 2006/12/26 13:13:23 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -140,14 +140,16 @@ static int imapdriver_login_sasl(mailsession * session,
 
 static int imapdriver_remove_message(mailsession * session, uint32_t num);
 
+static int imapdriver_parameters(mailsession * session,
+    int id, void * value);
+
 static mailsession_driver local_imap_session_driver = {
-#if ! defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
   /* sess_name */ "imap",
 
   /* sess_initialize */ imapdriver_initialize,
   /* sess_uninitialize */ imapdriver_uninitialize,
 
-  /* sess_parameters */ NULL,
+  /* sess_parameters */ imapdriver_parameters,
 
   /* sess_connect_stream */ imapdriver_connect_stream,
   /* sess_connect_path */ NULL,
@@ -184,59 +186,8 @@ static mailsession_driver local_imap_session_driver = {
   /* sess_get_messages_list */ imapdriver_get_messages_list,
   /* sess_get_envelopes_list */ imapdriver_get_envelopes_list,
   /* sess_remove_message */ imapdriver_remove_message,
-#if 0
-  /* sess_search_messages */ imapdriver_search_messages,
-#endif
-  /* sess_login_sasl */ imapdriver_login_sasl,
-  
-#else
-  .sess_name = "imap",
 
-  .sess_initialize = imapdriver_initialize,
-  .sess_uninitialize = imapdriver_uninitialize,
-
-  .sess_parameters = NULL,
-
-  .sess_connect_stream = imapdriver_connect_stream,
-  .sess_connect_path = NULL,
-  .sess_starttls = imapdriver_starttls,
-  .sess_login = imapdriver_login,
-  .sess_logout = imapdriver_logout,
-  .sess_noop = imapdriver_noop,
-
-  .sess_build_folder_name = imapdriver_build_folder_name,
-  .sess_create_folder = imapdriver_create_folder,
-  .sess_delete_folder = imapdriver_delete_folder,
-  .sess_rename_folder = imapdriver_rename_folder,
-  .sess_check_folder = imapdriver_check_folder,
-  .sess_examine_folder = imapdriver_examine_folder,
-  .sess_select_folder = imapdriver_select_folder,
-  .sess_expunge_folder = imapdriver_expunge_folder,
-  .sess_status_folder = imapdriver_status_folder,
-  .sess_messages_number = imapdriver_messages_number,
-  .sess_recent_number = imapdriver_recent_number,
-  .sess_unseen_number = imapdriver_unseen_number,
-  .sess_list_folders = imapdriver_list_folders,
-  .sess_lsub_folders = imapdriver_lsub_folders,
-  .sess_subscribe_folder = imapdriver_subscribe_folder,
-  .sess_unsubscribe_folder = imapdriver_unsubscribe_folder,
-
-  .sess_append_message = imapdriver_append_message,
-  .sess_append_message_flags = imapdriver_append_message_flags,
-  .sess_copy_message = imapdriver_copy_message,
-  .sess_move_message = NULL,
-
-  .sess_get_messages_list = imapdriver_get_messages_list,
-  .sess_get_envelopes_list = imapdriver_get_envelopes_list,
-  .sess_remove_message = imapdriver_remove_message,
-#if 0
-  .sess_search_messages = imapdriver_search_messages,
-#endif
-  
-  .sess_get_message = imapdriver_get_message,
-  .sess_get_message_by_uid = imapdriver_get_message_by_uid,
-  .sess_login_sasl = imapdriver_login_sasl,
-#endif
+  /* sess_login_sasl */ imapdriver_login_sasl
 };
 
 mailsession_driver * imap_session_driver = &local_imap_session_driver;
@@ -1165,7 +1116,9 @@ static int imapdriver_starttls(mailsession * session)
   mailstream_low * low;
   mailstream_low * new_low;
   int capability_available;
+  struct imap_session_state_data * data;
   
+  data = get_data(session);
   imap = get_imap_session(session);
 
   capability_available = FALSE;
@@ -1205,6 +1158,7 @@ static int imapdriver_starttls(mailsession * session)
   if (!starttls)
     return MAIL_ERROR_NO_TLS;
   
+#if 0
   r = mailimap_starttls(imap);
 
   switch (r) {
@@ -1219,7 +1173,8 @@ static int imapdriver_starttls(mailsession * session)
   if (fd == -1)
     return MAIL_ERROR_STREAM;
   
-  new_low = mailstream_low_tls_open(fd);
+  new_low = mailstream_low_tls_open_with_callback(fd,
+      data->imap_ssl_callback, data->imap_ssl_cb_data);
   if (new_low == NULL)
     return MAIL_ERROR_STREAM;
 
@@ -1227,6 +1182,9 @@ static int imapdriver_starttls(mailsession * session)
   mailstream_set_low(imap->imap_stream, new_low);
   
   return MAIL_NO_ERROR;
+#endif
+  r = mailimap_socket_starttls(imap);
+  return imap_error_to_mail_error(r);
 }
 
 static int imapdriver_get_message(mailsession * session,
@@ -1333,3 +1291,22 @@ err:
 	if (flags != NULL) mail_flags_free( flags);
 	return res;
 }
+
+static int imapdriver_parameters(mailsession * session,
+    int id, void * value)
+{
+  struct imap_session_state_data * data;
+  
+  data = get_data(session);
+  
+  switch (id) {
+  case IMAPDRIVER_CACHED_SET_SSL_CALLBACK:
+    data->imap_ssl_callback = value;
+    break;
+  case IMAPDRIVER_CACHED_SET_SSL_CALLBACK_DATA:
+    data->imap_ssl_cb_data = value;
+    break;
+  }
+  
+  return MAIL_ERROR_INVAL;
+}
