@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: imapdriver_cached.c,v 1.53 2006/06/26 11:50:26 hoa Exp $
+ * $Id: imapdriver_cached.c,v 1.56 2006/12/26 13:13:23 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -486,9 +486,10 @@ static int imapdriver_cached_parameters(mailsession * session,
       return r;
 
     return MAIL_NO_ERROR;
+    
+  default:
+    return mailsession_parameters(data->imap_ancestor, id, value);
   }
-
-  return MAIL_ERROR_INVAL;
 }
 
 
@@ -895,7 +896,7 @@ static int cmp_uid(uint32_t ** pa, uint32_t ** pb)
 }
 
 
-static void get_uid_from_filename(const char * filename)
+static void get_uid_from_filename(char * filename)
 {
   char * p;
   
@@ -1013,7 +1014,7 @@ static int boostrap_cache(mailsession * session)
     cache_item = malloc(sizeof(* cache_item));
     if (cache_item == NULL) {
       res = MAIL_ERROR_MEMORY;
-      goto free_keys;
+      goto free_keys_uid;
     }
     
     cache_item->uid = index;
@@ -1026,10 +1027,11 @@ static int boostrap_cache(mailsession * session)
   chash_free(keys);
   
   mail_cache_db_close_unlock(filename, cache_db);
+  mmap_string_free(mmapstr);
   
   return MAIL_NO_ERROR;
   
-/* free_keys_uid:*/
+ free_keys_uid:
   chash_free(keys_uid);
  free_keys:
   chash_free(keys);
@@ -1046,9 +1048,6 @@ static int imapdriver_cached_get_messages_list(mailsession * session,
 					       struct mailmessage_list **
 					       result)
 {
-#if 0
-  mailsession * imap_session;
-#endif
   mailimap * imap;
   uint32_t uid_max;
   struct imap_cached_session_state_data * data;
@@ -1058,11 +1057,6 @@ static int imapdriver_cached_get_messages_list(mailsession * session,
   int res;
   carray * tab;
 
-#if 0  
-  data = session->data;
-  imap_session = get_ancestor(session);
-  imap = ((struct imap_session_state_data *) (imap_session->data))->session;
-#endif
   data = get_cached_data(session);
   imap = get_imap_session(session);
 
