@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: imapdriver_cached_message.c,v 1.25 2006/06/26 11:50:26 hoa Exp $
+ * $Id: imapdriver_cached_message.c,v 1.26 2006/11/12 15:15:58 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -285,6 +285,9 @@ static void generate_key_from_message(char * key, size_t size,
   case MAILIMAP_MSG_ATT_ENVELOPE:
     snprintf(key, size, "%s-envelope", msg_info->msg_uid);
     break;
+  case MAILIMAP_MSG_ATT_BODYSTRUCTURE:
+    snprintf(key, size, "%s-bodystructure", msg_info->msg_uid);
+    break;
   }
 }
 
@@ -458,10 +461,46 @@ static int imap_fetch_size(mailmessage * msg_info,
   return mailmessage_fetch_size(get_ancestor(msg_info), result);
 }
 
+static void cleanup_mime(struct mailmime * mime)
+{
+  mime->mm_mime_start = NULL;
+  mime->mm_length = 0;
+  if (mime->mm_body != NULL) {
+    mailmime_data_free(mime->mm_body);
+    mime->mm_body = NULL;
+  }
+  switch (mime->mm_type) {
+  case MAILMIME_SINGLE:
+    mime->mm_data.mm_single = NULL;
+    break;
+  case MAILMIME_MULTIPLE:
+    {
+      clistiter * cur;
+      
+      for(cur = clist_begin(mime->mm_data.mm_multipart.mm_mp_list) ;
+          cur != NULL ; cur = clist_next(cur)) {
+        struct mailmime * submime;
+        
+        submime = clist_content(cur);
+        cleanup_mime(submime);
+      }
+    }
+    break;
+  case MAILMIME_MESSAGE:
+    if (mime->mm_data.mm_message.mm_msg_mime != NULL) 
+      cleanup_mime(mime->mm_data.mm_message.mm_msg_mime);
+    break;
+  }
+}
+
 static int imap_get_bodystructure(mailmessage * msg_info,
 				  struct mailmime ** result)
 {
   int r;
+  char key[PATH_MAX];
+  char filename[PATH_MAX];
+  char * str;
+  size_t len;
   
   if (msg_info->msg_mime != NULL) {
     * result = msg_info->msg_mime;
@@ -469,11 +508,48 @@ static int imap_get_bodystructure(mailmessage * msg_info,
     return MAIL_NO_ERROR;
   }
   
+  generate_key_from_message(key, PATH_MAX,
+      msg_info, MAILIMAP_MSG_ATT_BODYSTRUCTURE);
+  
+  build_cache_name(filename, PATH_MAX, msg_info, key);
+  
+  r = generic_cache_read(filename, &str, &len);
+  if (r == MAIL_NO_ERROR) {
+    size_t cur_index;
+    struct mailmime * mime;
+    
+    cur_index = 0;
+    r = mailmime_parse(str, len, &cur_index, &mime);
+    
+    mmap_string_unref(str);
+    
+    cleanup_mime(mime);
+    
+    msg_info->msg_mime = mime;
+    
+    return MAIL_NO_ERROR;
+  }
+  
   r = mailmessage_get_bodystructure(get_ancestor(msg_info),
       result);
   if (r == MAIL_NO_ERROR) {
+    int col;
+    FILE * f;
+    
     msg_info->msg_mime = get_ancestor(msg_info)->msg_mime;
     get_ancestor(msg_info)->msg_mime = NULL;
+    
+    f = fopen(filename, "w");
+    if (f == NULL) {
+      return MAIL_ERROR_FILE;
+    }
+    col = 0;
+    r = mailmime_write(f, &col, msg_info->msg_mime);
+    if (r != MAILIMF_NO_ERROR) {
+      fclose(f);
+      return MAIL_ERROR_FILE;
+    }
+    fclose(f);
   }
   
   return r;
