@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: imapdriver_tools.c,v 1.22 2006/05/22 13:39:40 hoa Exp $
+ * $Id: imapdriver_tools.c,v 1.24 2006/12/13 18:31:31 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -195,7 +195,10 @@ int imap_error_to_mail_error(int error)
 
   case MAILIMAP_ERROR_STARTTLS:
     return MAIL_ERROR_STARTTLS;
-
+    
+  case MAILIMAP_ERROR_SSL:
+    return MAIL_ERROR_SSL;
+    
   case MAILIMAP_ERROR_INVAL:
     return MAIL_ERROR_INVAL;
 
@@ -914,27 +917,36 @@ imap_body_fields_to_mime_fields(struct mailimap_body_fields * body_fields,
   }
 
   if (imap_lang != NULL) {
-    r = imap_language_to_mime_language(imap_lang, &lang);
-    if (r != MAIL_NO_ERROR) {
-      res = MAIL_ERROR_MEMORY;
-      goto free_list;
-    }
+    int has_lang;
+    
+    has_lang = 1;
+    if (imap_lang->lg_type == MAILIMAP_BODY_FLD_LANG_SINGLE)
+      if (imap_lang->lg_data.lg_single == NULL)
+        has_lang = 0;
+    
+    if (has_lang) {
+      r = imap_language_to_mime_language(imap_lang, &lang);
+      if (r != MAIL_NO_ERROR) {
+        res = MAIL_ERROR_MEMORY;
+        goto free_list;
+      }
+      
+      type = MAILMIME_FIELD_LANGUAGE;
 
-    type = MAILMIME_FIELD_LANGUAGE;
+      mime_field = mailmime_field_new(type, NULL,
+          NULL, NULL, NULL, 0, NULL, lang);
+      if (mime_field == NULL) {
+        mailmime_language_free(lang);
+        res = MAIL_ERROR_MEMORY;
+        goto free_list;
+      }
 
-    mime_field = mailmime_field_new(type, NULL,
-        NULL, NULL, NULL, 0, NULL, lang);
-    if (mime_field == NULL) {
-      mailmime_language_free(lang);
-      res = MAIL_ERROR_MEMORY;
-      goto free_list;
-    }
-
-    r = clist_append(list, mime_field);
-    if (r != 0) {
-      mailmime_field_free(mime_field);
-      res = MAIL_ERROR_MEMORY;
-      goto free_list;
+      r = clist_append(list, mime_field);
+      if (r != 0) {
+        mailmime_field_free(mime_field);
+        res = MAIL_ERROR_MEMORY;
+        goto free_list;
+      }
     }
   }
 
