@@ -30,15 +30,19 @@
  */
 
 /*
- * $Id: mboxdriver_tools.c,v 1.15 2005/06/01 12:21:58 smarinier Exp $
+ * $Id: mboxdriver_tools.c,v 1.16 2006/05/22 13:39:40 hoa Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#	include <config.h>
+#endif
+
 #include "mboxdriver_tools.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #	include <unistd.h>
 #endif
 
