@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mhdriver.c,v 1.34 2006/06/26 11:50:26 hoa Exp $
+ * $Id: mhdriver.c,v 1.35 2006/12/13 18:31:31 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -384,11 +384,21 @@ static int get_parent(mailsession * session, const char * mb,
     return MAIL_ERROR_BAD_STATE;
 
   length = strlen(mb);
-  for(i = length - 1 ; i >= 0 ; i--)
-    if (mb[i] == '/')
-      break;
-  name = mb + i + 1;
-
+  if (length > 0) {
+    i = length - 1;
+    while (1) {
+      if (i == 0)
+        return MAIL_ERROR_INVAL;
+      if (mb[i] == '/')
+        break;
+      i --;
+    }
+    name = mb + i + 1;
+  }
+  else {
+    return MAIL_ERROR_INVAL;
+  }
+  
   parent_name = malloc(i + 1);
   /* strndup(mb, i) */
   if (parent_name == NULL)
@@ -412,7 +422,7 @@ static int mhdriver_create_folder(mailsession * session, const char * mb)
 {
   int r;
   struct mailmh_folder * parent;
-  char * name;
+  const char * name;
   
   r = get_parent(session, mb, &parent, &name);
   if (r != MAIL_NO_ERROR)
@@ -450,7 +460,7 @@ static int mhdriver_rename_folder(mailsession * session, const char * mb,
 {
   struct mailmh_folder * src_folder;
   struct mailmh_folder * dst_folder;
-  char * name;
+  const char * name;
   struct mailmh * mh;
   int r;
 
