@@ -30,16 +30,15 @@
  */
 
 /*
- * $Id: nntpdriver_cached.c,v 1.50 2005/08/15 11:08:40 hoa Exp $
+ * $Id: nntpdriver_cached.c,v 1.53 2006/06/26 11:50:27 hoa Exp $
  */
 
-#include "nntpdriver_cached.h"
-
-#ifndef CONFIG_H
-#define CONFIG_H
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#	include <config.h>
 #endif
 
+#include "nntpdriver_cached.h"
+
 #include "libetpan-config.h"
 
 #include <string.h>
@@ -47,7 +46,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #	include <unistd.h>
 #endif
 #include <stdlib.h>
@@ -80,34 +79,34 @@ static int nntpdriver_cached_connect_stream(mailsession * session,
 					    mailstream * s);
 
 static int nntpdriver_cached_login(mailsession * session,
-				   char * userid, char * password);
+				   const char * userid, const char * password);
 
 static int nntpdriver_cached_logout(mailsession * session);
 
 static int nntpdriver_cached_check_folder(mailsession * session);
 
-static int nntpdriver_cached_select_folder(mailsession * session, char * mb);
+static int nntpdriver_cached_select_folder(mailsession * session, const char * mb);
 
 static int nntpdriver_cached_status_folder(mailsession * session,
-					   char * mb,
+					   const char * mb,
 					   uint32_t * result_messages,
 					   uint32_t * result_recent,
 					   uint32_t * result_unseen);
 
-static int nntpdriver_cached_messages_number(mailsession * session, char * mb,
+static int nntpdriver_cached_messages_number(mailsession * session, const char * mb,
 					     uint32_t * result);
 
-static int nntpdriver_cached_recent_number(mailsession * session, char * mb,
+static int nntpdriver_cached_recent_number(mailsession * session, const char * mb,
 					   uint32_t * result);
 
-static int nntpdriver_cached_unseen_number(mailsession * session, char * mb,
+static int nntpdriver_cached_unseen_number(mailsession * session, const char * mb,
 					   uint32_t * result);
 
 static int nntpdriver_cached_append_message(mailsession * session,
-					    char * message, size_t size);
+					    const char * message, size_t size);
 
 static int nntpdriver_cached_append_message_flags(mailsession * session,
-    char * message, size_t size, struct mail_flags * flags);
+    const char * message, size_t size, struct mail_flags * flags);
 
 static int
 nntpdriver_cached_get_envelopes_list(mailsession * session,
@@ -118,17 +117,17 @@ static int
 nntpdriver_cached_get_messages_list(mailsession * session,
 				    struct mailmessage_list ** result);
 
-static int nntpdriver_cached_list_folders(mailsession * session, char * mb,
+static int nntpdriver_cached_list_folders(mailsession * session, const char * mb,
 					  struct mail_list ** result);
 
-static int nntpdriver_cached_lsub_folders(mailsession * session, char * mb,
+static int nntpdriver_cached_lsub_folders(mailsession * session, const char * mb,
 					  struct mail_list ** result);
 
 static int nntpdriver_cached_subscribe_folder(mailsession * session,
-					      char * mb);
+					      const char * mb);
 
 static int nntpdriver_cached_unsubscribe_folder(mailsession * session,
-						char * mb);
+						const char * mb);
 
 static int nntpdriver_cached_get_message(mailsession * session,
 					 uint32_t num, mailmessage ** result);
@@ -140,7 +139,7 @@ static int nntpdriver_cached_get_message_by_uid(mailsession * session,
     mailmessage ** result);
 
 static mailsession_driver local_nntp_cached_session_driver = {
-#if defined (_MSC_VER) || defined (__SUNPRO_CC)
+#if ! defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
   /* sess_name */ "nntp-cached",
 
   /* sess_initialize */ nntpdriver_cached_initialize,
@@ -418,7 +417,7 @@ static int nntpdriver_cached_connect_stream(mailsession * session,
 }
 
 static int nntpdriver_cached_login(mailsession * session,
-				   char * userid, char * password)
+				   const char * userid, const char * password)
 {
   return mailsession_login(get_ancestor(session), userid, password);
 }
@@ -438,7 +437,7 @@ static int nntpdriver_cached_logout(mailsession * session)
   return mailsession_logout(get_ancestor(session));
 }
 
-static int nntpdriver_cached_select_folder(mailsession * session, char * mb)
+static int nntpdriver_cached_select_folder(mailsession * session, const char * mb)
 {
   int r;
   struct nntp_session_state_data * ancestor_data;
@@ -501,7 +500,7 @@ static int nntpdriver_cached_check_folder(mailsession * session)
 
 
 static int nntpdriver_cached_status_folder(mailsession * session,
-    char * mb, uint32_t * result_messages, uint32_t * result_recent,
+    const char * mb, uint32_t * result_messages, uint32_t * result_recent,
     uint32_t * result_unseen)
 {
   int res;
@@ -607,7 +606,7 @@ static int nntpdriver_cached_status_folder(mailsession * session,
 }
 
 static int nntpdriver_cached_messages_number(mailsession * session,
-					     char * mb,
+					     const char * mb,
 					     uint32_t * result)
 {
   uint32_t messages;
@@ -626,7 +625,7 @@ static int nntpdriver_cached_messages_number(mailsession * session,
 }
 
 static int nntpdriver_cached_recent_number(mailsession * session,
-					   char * mb,
+					   const char * mb,
 					   uint32_t * result)
 {
   uint32_t messages;
@@ -645,7 +644,7 @@ static int nntpdriver_cached_recent_number(mailsession * session,
 }
 
 static int nntpdriver_cached_unseen_number(mailsession * session,
-					   char * mb,
+					   const char * mb,
 					   uint32_t * result)
 {
   uint32_t messages;
@@ -663,26 +662,26 @@ static int nntpdriver_cached_unseen_number(mailsession * session,
   return MAIL_NO_ERROR;  
 }
 
-static int nntpdriver_cached_list_folders(mailsession * session, char * mb,
+static int nntpdriver_cached_list_folders(mailsession * session, const char * mb,
 					  struct mail_list ** result)
 {
   return mailsession_list_folders(get_ancestor(session), mb, result);
 }
 
-static int nntpdriver_cached_lsub_folders(mailsession * session, char * mb,
+static int nntpdriver_cached_lsub_folders(mailsession * session, const char * mb,
 					  struct mail_list ** result)
 {
   return mailsession_lsub_folders(get_ancestor(session), mb, result);
 }
 
 static int nntpdriver_cached_subscribe_folder(mailsession * session,
-					      char * mb)
+					      const char * mb)
 {
   return mailsession_subscribe_folder(get_ancestor(session), mb);
 }
 
 static int nntpdriver_cached_unsubscribe_folder(mailsession * session,
-						char * mb)
+						const char * mb)
 {
   return mailsession_unsubscribe_folder(get_ancestor(session), mb);
 }
@@ -692,13 +691,13 @@ static int nntpdriver_cached_unsubscribe_folder(mailsession * session,
 /* messages operations */
 
 static int nntpdriver_cached_append_message(mailsession * session,
-					    char * message, size_t size)
+					    const char * message, size_t size)
 {
   return mailsession_append_message(get_ancestor(session), message, size);
 }
 
 static int nntpdriver_cached_append_message_flags(mailsession * session,
-    char * message, size_t size, struct mail_flags * flags)
+    const char * message, size_t size, struct mail_flags * flags)
 {
   return nntpdriver_cached_append_message(session, message, size);
 }
