@@ -30,18 +30,17 @@
  */
 
 /*
- * $Id: maildriver_tools.c,v 1.32 2005/06/01 20:23:15 hoa Exp $
+ * $Id: maildriver_tools.c,v 1.34 2006/06/07 15:10:01 smarinier Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#	include <config.h>
+#endif
+
 #include "maildriver_tools.h"
 
 #include "libetpan-config.h"
 
-#ifndef CONFIG_H
-#define CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
@@ -50,7 +49,7 @@
 #else
 #	include <dirent.h>
 #endif
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #	include <unistd.h>
 #endif
 #include "maildriver.h"
@@ -645,7 +644,7 @@ int maildriver_imf_error_to_mail_error(int error)
   }
 }
 
-char * maildriver_quote_mailbox(char * mb)
+char * maildriver_quote_mailbox(const char * mb)
 {
   MMAPString * gstr;
   char * str;
