@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailstorage_tools.c,v 1.19 2006/05/22 13:39:40 hoa Exp $
+ * $Id: mailstorage_tools.c,v 1.21 2006/12/22 18:45:44 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,10 +53,12 @@
 #endif
 #include <fcntl.h>
 #include <string.h>
+#include <stdio.h>
 
 #include "mail.h"
 #include "mailmessage.h"
 #include "maildriver.h"
+#include "connect.h"
 
 /* tools */
 
@@ -64,44 +66,9 @@
 
 static int tcp_connect(char * server, uint16_t port)
 {
-  struct hostent * remotehost;
-  struct sockaddr_in sa;
-  int r;
-
-#ifdef WIN32
-  SOCKET s;
-#else
-  int s;
-#endif
-  s = socket(PF_INET, SOCK_STREAM, 0);
-  if (s == -1)
-    goto err;
-
-  remotehost = gethostbyname(server);
-  if (remotehost == NULL)
-    goto close_socket;
-
-  sa.sin_family = AF_INET;
-  sa.sin_port = htons(port);
-  memcpy(&sa.sin_addr, remotehost->h_addr, remotehost->h_length);
-
-  r = connect(s, (struct sockaddr *) &sa, sizeof(struct sockaddr_in));
-  if (r == -1)
-    goto close_socket;
-
-  return s;
-
- close_socket:
-#ifdef WIN32
-  closesocket( s);
-#else
-  close(s);
-#endif
- err:
-  return -1;
+  return mail_tcp_connect(server, port);
 }
 
-
 /* connection through a shell command */
 /* SEB unsupported on Windows */
 #ifndef WIN32
