@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailthread.c,v 1.28 2006/05/22 13:39:40 hoa Exp $
+ * $Id: mailthread.c,v 1.29 2006/12/26 13:13:24 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -479,65 +479,6 @@ static int get_thread_subject(char * default_from,
 
 
 
-#ifndef WRONG
-#define WRONG	(-1)
-#endif /* !defined WRONG */
-
-static int tmcomp(struct tm * atmp, struct tm * btmp)
-{
-  register int	result;
-  
-  if ((result = (atmp->tm_year - btmp->tm_year)) == 0 &&
-      (result = (atmp->tm_mon - btmp->tm_mon)) == 0 &&
-      (result = (atmp->tm_mday - btmp->tm_mday)) == 0 &&
-      (result = (atmp->tm_hour - btmp->tm_hour)) == 0 &&
-      (result = (atmp->tm_min - btmp->tm_min)) == 0)
-    result = atmp->tm_sec - btmp->tm_sec;
-  return result;
-}
-
-static time_t mkgmtime(struct tm * tmp)
-{
-  register int			dir;
-  register int			bits;
-  register int			saved_seconds;
-  time_t				t;
-  struct tm			yourtm, mytm;
-  
-  yourtm = *tmp;
-  saved_seconds = yourtm.tm_sec;
-  yourtm.tm_sec = 0;
-  /*
-  ** Calculate the number of magnitude bits in a time_t
-  ** (this works regardless of whether time_t is
-  ** signed or unsigned, though lint complains if unsigned).
-  */
-  for (bits = 0, t = 1; t > 0; ++bits, t <<= 1)
-    ;
-  /*
-  ** If time_t is signed, then 0 is the median value,
-  ** if time_t is unsigned, then 1 << bits is median.
-  */
-  t = (t < 0) ? 0 : ((time_t) 1 << bits);
-  for ( ; ; ) {
-    gmtime_r(&t, &mytm);
-    dir = tmcomp(&mytm, &yourtm);
-    if (dir != 0) {
-      if (bits-- < 0)
-	return WRONG;
-      if (bits < 0)
-	--t;
-      else if (dir > 0)
-	t -= (time_t) 1 << bits;
-      else	t += (time_t) 1 << bits;
-      continue;
-    }
-    break;
-  }
-  t += saved_seconds;
-  return t;
-}
-
 static inline time_t get_date(mailmessage * msg)
 {
   struct tm tmval;
@@ -557,7 +498,7 @@ static inline time_t get_date(mailmessage * msg)
   tmval.tm_mon  = date_time->dt_month - 1;
   tmval.tm_year = date_time->dt_year - 1900;
   
-  timeval = mkgmtime(&tmval);
+  timeval = mail_mkgmtime(&tmval);
   
   timeval -= date_time->dt_zone * 36;
   
