@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailprivacy_gnupg.c,v 1.8 2005/12/07 14:57:57 hoa Exp $
+ * $Id: mailprivacy_gnupg.c,v 1.10 2006/06/26 11:50:27 hoa Exp $
  */
 
 /* passphrase is needed when private key is needed
@@ -39,6 +39,10 @@
    - and to decrypt a message
 */
 
+#ifdef HAVE_CONFIG_H
+#	include <config.h>
+#endif
+
 #include "mailprivacy_gnupg.h"
 
 #include "mailprivacy.h"
@@ -64,7 +68,7 @@ enum {
   ERROR_PGP_CHECK,
   ERROR_PGP_COMMAND,
   ERROR_PGP_FILE,
-  ERROR_PGP_NOPASSPHRASE,
+  ERROR_PGP_NOPASSPHRASE
 };
 
 static int mailprivacy_gnupg_add_encryption_id(struct mailprivacy * privacy,
@@ -323,7 +327,7 @@ static int get_pgp_output(FILE * dest_f, char * command)
 
 enum {
   STATE_USERID,
-  STATE_NORMAL,
+  STATE_NORMAL
 };
 
 static int get_userid(char * filename, char * username, size_t length)
@@ -2809,7 +2813,7 @@ static int pgp_armor_sign_encrypt(struct mailprivacy * privacy,
   return res;
 }
 
-#if defined (_MSC_VER) || defined ( __SUNPRO_CC)
+#if ! defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
 static struct mailprivacy_encryption pgp_encryption_tab[] = {
   /* PGP signed part */
   {
@@ -2909,7 +2913,7 @@ static struct mailprivacy_encryption pgp_encryption_tab[] = {
 };
 #endif
 
-#if defined (_MSC_VER) || defined ( __SUNPRO_CC)
+#if ! defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
 static struct mailprivacy_protocol pgp_protocol = {
   /* name */ "pgp",
   /* description */ "OpenPGP",
