@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailprivacy_smime.c,v 1.14 2006/06/26 11:50:27 hoa Exp $
+ * $Id: mailprivacy_smime.c,v 1.15 2006/08/29 22:27:35 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1354,6 +1354,8 @@ static char * get_file(chash * hash, char * email)
   return data.data;
 }
 
+#define CERTIFICATE_SUFFIX "-cert.pem"
+
 void mailprivacy_smime_set_cert_dir(struct mailprivacy * privacy,
     char * directory)
 {
@@ -1376,6 +1378,7 @@ void mailprivacy_smime_set_cert_dir(struct mailprivacy * privacy,
     return;
   
   while ((ent = readdir(dir)) != NULL) {
+#if 0
     char quoted_filename[PATH_MAX];
     char filename[PATH_MAX];
     char command[PATH_MAX];
@@ -1399,6 +1402,30 @@ void mailprivacy_smime_set_cert_dir(struct mailprivacy * privacy,
       set_file(certificates, buf, filename);
     
     pclose(p);
+#endif
+    char filename[PATH_MAX];
+    char email[PATH_MAX];
+    char * p;
+    
+    snprintf(filename, sizeof(filename),
+        "%s/%s", directory, ent->d_name);
+    
+    strncpy(email, ent->d_name, sizeof(email));
+    email[sizeof(email) - 1] = '\0';
+    
+    p = strstr(email, CERTIFICATE_SUFFIX);
+    if (p == NULL)
+      continue;
+    
+    if (strlen(p) != sizeof(CERTIFICATE_SUFFIX) - 1)
+      continue;
+    
+    * p = 0;
+    
+    if (* email == '\0')
+      continue;
+    
+    set_file(certificates, email, filename);
   }
   closedir(dir);
 }
@@ -1563,7 +1590,7 @@ static int get_cert_from_sig(struct mailprivacy * privacy,
   }
   
   snprintf(store_cert_filename, sizeof(store_cert_filename),
-      "%s/%s-cert.pem", cert_dir, email);
+      "%s/%s" CERTIFICATE_SUFFIX, cert_dir, email);
   
   r = mail_quote_filename(quoted_store_cert_filename,
        sizeof(quoted_store_cert_filename), store_cert_filename);
@@ -1571,7 +1598,7 @@ static int get_cert_from_sig(struct mailprivacy * privacy,
     res = MAIL_ERROR_MEMORY;
     goto unlink_signature;
   }
-
+  
   snprintf(command, sizeof(command),
       "openssl pkcs7 -inform DER -in '%s' -out '%s' -print_certs 2>/dev/null",
       quoted_signature_filename, quoted_store_cert_filename);
