@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailmime_decode.c,v 1.30 2006/10/07 17:04:18 hoa Exp $
+ * $Id: mailmime_decode.c,v 1.31 2006/12/12 19:44:29 hoa Exp $
  */
 
 /*
@@ -476,7 +476,12 @@ static int mailmime_encoded_word_parse(const char * message, size_t length,
       goto free_encoded_text;
     }
   }
-
+  
+  /* fix charset */
+  if (strcasecmp(charset, "utf8") == 0) {
+    free(charset);
+    charset = strdup("utf-8");
+  }
   ew = mailmime_encoded_word_new(charset, text);
   if (ew == NULL) {
     res = MAILIMF_ERROR_MEMORY;
