@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailpop3.c,v 1.28 2006/08/29 23:14:02 hoa Exp $
+ * $Id: mailpop3.c,v 1.29 2006/12/22 18:45:44 hoa Exp $
  */
 
 /*
@@ -455,8 +455,8 @@ int mailpop3_apop(mailpop3 * f,
   /* calculate md5 sum */
 
   MD5Init(&md5context);
-  MD5Update(&md5context, f->pop3_timestamp, strlen (f->pop3_timestamp));
-  MD5Update(&md5context, password, strlen (password));
+  MD5Update(&md5context, (const unsigned char *) f->pop3_timestamp, strlen (f->pop3_timestamp));
+  MD5Update(&md5context, (const unsigned char *) password, strlen (password));
   MD5Final(md5digest, &md5context);
   
   cmd_ptr = md5string;
