@@ -32,7 +32,7 @@
  */
 
 /*
- * $Id: mailsmtp.c,v 1.25 2006/05/22 13:39:42 hoa Exp $
+ * $Id: mailsmtp.c,v 1.28 2006/12/13 18:31:33 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -180,18 +180,27 @@ int mailsmtp_quit(mailsmtp * session)
 {
   char command[SMTP_STRING_SIZE];
   int r;
-
+  int res;
+  
   snprintf(command, SMTP_STRING_SIZE, "QUIT\r\n");
   r = send_command(session, command);
-  if (r == -1)
-    return MAILSMTP_ERROR_STREAM;
+  if (r == -1) {
+    res = MAILSMTP_ERROR_STREAM;
+    goto close;
+  }
   r = read_response(session);
-  if (r == 0)
-    return MAILSMTP_ERROR_STREAM;
+  if (r == 0) {
+    res = MAILSMTP_ERROR_STREAM;
+    goto close;
+  }
+  
+  res = MAILSMTP_NO_ERROR;  
+  
+ close:
   mailstream_close(session->stream);
   session->stream = NULL;
   
-  return MAILSMTP_NO_ERROR;
+  return res;
 }
 
 
@@ -617,6 +626,8 @@ int auth_map_errors(int err)
     return MAILSMTP_ERROR_AUTH_REQUIRED;
   case 534:
     return MAILSMTP_ERROR_AUTH_TOO_WEAK;
+  case 535:
+    return MAILSMTP_ERROR_AUTH_AUTHENTICATION_FAILED;
   case 538:
     return MAILSMTP_ERROR_AUTH_ENCRYPTION_REQUIRED;
   default:
@@ -932,7 +943,7 @@ int mailesmtp_starttls(mailsmtp * session) {
 }
 
 static int parse_response(mailsmtp * session,
-			   char * response)
+    char * response)
 {
   char * message;
   int code;
@@ -1177,7 +1188,13 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
   session->smtp_sasl.sasl_secret = secret;
   
   /* init SASL */
-  mailsasl_ref();
+  if (session->smtp_sasl.sasl_conn != NULL) {
+    sasl_dispose((sasl_conn_t **) &session->smtp_sasl.sasl_conn);
+    session->smtp_sasl.sasl_conn = NULL;
+  }
+  else {
+    mailsasl_ref();
+  }
   
   r = sasl_client_new("smtp", server_fqdn,
       local_ip_port, remote_ip_port, sasl_callback, 0,
@@ -1191,7 +1208,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
       auth_type, NULL, &sasl_out, &sasl_out_len, &mechusing);
   if ((r != SASL_CONTINUE) && (r != SASL_OK)) {
     res = MAILSMTP_ERROR_AUTH_LOGIN;
-    goto free_secret;
+    goto free_sasl_conn;
   }
   
   if (sasl_out_len != 0) {
@@ -1199,7 +1216,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
     encoded = malloc(max_encoded + 1);
     if (encoded == NULL) {
       res = MAILSMTP_ERROR_MEMORY;
-      goto free_secret;
+      goto free_sasl_conn;
     }
     
     r = sasl_encode64(sasl_out, sasl_out_len,
@@ -1207,7 +1224,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
     if (r != SASL_OK) {
       free(encoded);
       res = MAILSMTP_ERROR_MEMORY;
-      goto free_secret;
+      goto free_sasl_conn;
     }
     
     snprintf(command, SMTP_STRING_SIZE, "AUTH %s %s\r\n", auth_type, encoded);
@@ -1221,7 +1238,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
   r = send_command(session, command);
   if (r == -1) {
     res = MAILSMTP_ERROR_STREAM;
-    goto free_secret;
+    goto free_sasl_conn;
   }
   
   while (1) {
@@ -1229,9 +1246,9 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
     switch (r) {
     case 220:
     case 235:
-      free(session->smtp_sasl.sasl_secret);
-      return MAILSMTP_NO_ERROR;
-    
+      res = MAILSMTP_NO_ERROR;
+      goto free_sasl_conn;
+      
     case 334:
       {
         size_t response_len;
@@ -1244,7 +1261,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
         decoded = malloc(max_decoded + 1);
         if (decoded == NULL) {
           res = MAILSMTP_ERROR_MEMORY;
-          goto free_secret;
+          goto free_sasl_conn;
         }
         
         r = sasl_decode64(session->response, response_len,
@@ -1253,7 +1270,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
         if (r != SASL_OK) {
           free(decoded);
           res = MAILSMTP_ERROR_MEMORY;
-          goto free_secret;
+          goto free_sasl_conn;
         }
         
         r = sasl_client_step(session->smtp_sasl.sasl_conn,
@@ -1263,14 +1280,14 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
         
         if ((r != SASL_CONTINUE) && (r != SASL_OK)) {
           res = MAILSMTP_ERROR_AUTH_LOGIN;
-          goto free_secret;
+          goto free_sasl_conn;
         }
         
         max_encoded = ((sasl_out_len + 2) / 3) * 4;
         encoded = malloc(max_encoded + 1);
         if (encoded == NULL) {
           res = MAILSMTP_ERROR_MEMORY;
-          goto free_secret;
+          goto free_sasl_conn;
         }
         
         r = sasl_encode64(sasl_out, sasl_out_len,
@@ -1278,7 +1295,7 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
         if (r != SASL_OK) {
           free(encoded);
           res = MAILSMTP_ERROR_MEMORY;
-          goto free_secret;
+          goto free_sasl_conn;
         }
         
         snprintf(command, SMTP_STRING_SIZE, "%s\r\n", encoded);
@@ -1288,21 +1305,26 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
         
         if (r == -1) {
           res = MAILSMTP_ERROR_STREAM;
-          goto free_secret;
+          goto free_sasl_conn;
         }
       }
       break;
       
     default:
       res = auth_map_errors(r);
-      goto free_secret;
+      goto free_sasl_conn;
     }
   }
   
-  return MAILSMTP_NO_ERROR;
+  res = MAILSMTP_NO_ERROR;
   
+ free_sasl_conn:
+  sasl_dispose((sasl_conn_t **) &session->smtp_sasl.sasl_conn);
+  session->smtp_sasl.sasl_conn = NULL;
+  mailsasl_unref();
  free_secret:
   free(session->smtp_sasl.sasl_secret);
+  session->smtp_sasl.sasl_secret = NULL;
  err:
   return res;
 #else
