@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailsmtp.h,v 1.17 2005/07/16 17:55:57 hoa Exp $
+ * $Id: mailsmtp.h,v 1.20 2005/12/20 17:56:11 hoa Exp $
  */
 
 #ifndef MAILSMTP_H
@@ -47,11 +47,17 @@ extern "C" {
 #include <libetpan/mailsmtp_ssl.h>
 
 
+LIBETPAN_EXPORT
 mailsmtp * mailsmtp_new(size_t progr_rate,
 			progress_function * progr_fun);
+
+LIBETPAN_EXPORT
 void mailsmtp_free(mailsmtp * session);
 
+LIBETPAN_EXPORT
 int mailsmtp_connect(mailsmtp * session, mailstream * s);
+
+LIBETPAN_EXPORT
 int mailsmtp_quit(mailsmtp * session);
 
 
@@ -60,6 +66,7 @@ int mailsmtp_quit(mailsmtp * session);
  * Tries AUTH with detected method - "better" method first:
  * CRAM-MD5 -> PLAIN -> LOGIN
  */
+LIBETPAN_EXPORT
 int mailsmtp_auth(mailsmtp * session, const char * user, const char * pass);
 
 /* This call is deprecated and mailesmtp_auth_sasl() should be used instead */
@@ -67,29 +74,49 @@ int mailsmtp_auth(mailsmtp * session, const char * user, const char * pass);
  * tries to autenticate with the server using given auth-type
  * returns MAILSMTP_NO_ERROR on success
  */
+LIBETPAN_EXPORT
 int mailsmtp_auth_type(mailsmtp * session,
     const char * user, const char * pass, int type);
 
+LIBETPAN_EXPORT
 int mailsmtp_helo(mailsmtp * session);
+
+LIBETPAN_EXPORT
 int mailsmtp_mail(mailsmtp * session, const char * from);
+
+LIBETPAN_EXPORT
 int mailsmtp_rcpt(mailsmtp * session, const char * to);
+
+LIBETPAN_EXPORT
 int mailsmtp_data(mailsmtp * session);
+
+LIBETPAN_EXPORT
 int mailsmtp_data_message(mailsmtp * session,
 			   const char * message,
 			   size_t size);
+
+LIBETPAN_EXPORT
 int mailesmtp_ehlo(mailsmtp * session);
+
+LIBETPAN_EXPORT
 int mailesmtp_mail(mailsmtp * session,
 		    const char * from,
 		    int return_full,
 		    const char * envid);
+
+LIBETPAN_EXPORT
 int mailesmtp_rcpt(mailsmtp * session,
 		    const char * to,
 		    int notify,
 		    const char * orcpt);
+
+LIBETPAN_EXPORT
 int mailesmtp_starttls(mailsmtp * session);
 
+LIBETPAN_EXPORT
 const char * mailsmtp_strerror(int errnum);
 
+LIBETPAN_EXPORT
 int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
     const char * server_fqdn,
     const char * local_ip_port,
@@ -97,6 +124,12 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
     const char * login, const char * auth_name,
     const char * password, const char * realm);
 
+LIBETPAN_EXPORT
+int mailsmtp_noop(mailsmtp * session);
+
+LIBETPAN_EXPORT
+int mailsmtp_reset(mailsmtp * session);
+
 #ifdef __cplusplus
 }
 #endif
